import os
dir_path = os.path.dirname(os.path.realpath(__file__))
import sys
sys.path.append(os.path.join(dir_path,'graphics'))
from graphics import shapes, funcs

class Shape(shapes.Image):
    def __init__(self, n):
        self.n = n
        self.direction = 0

    def image(self):
        return funcs.rotateImage((
            ((1, 1),
             (1, 1)),

            ((0, 1, 0),
             (1, 1, 1)),

            ((1, 0, 0),
             (1, 1, 1)),

            ((0, 0, 1),
             (1, 1, 1)),

            ((1, 1, 1, 1),),

            ((0, 1, 1),
             (1, 1, 0)),

            ((1, 1, 0),
             (0, 1, 1))
        )[self.n], self.direction)



import sys


def has_colors(stream):
    if not hasattr(stream, 'isatty'):
        return False
    if not stream.isatty():
        return False
    try:
        import curses
        curses.setupterm()
        return curses.tigetnum('colors') > 2
    except:
        return False

BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = list(range(8))
has_colors = has_colors(sys.stdout)


def colorStr(text, color=WHITE):
    if has_colors:
        seq = '\x1b[1;%dm' % (30 + color) + text + '\x1b[0m'
        return seq
    else:
        return text

if __name__ == '__main__':
    print(colorStr('hi', CYAN))



from rp import *
import mpi4py
from mpi4py import MPI
comm=MPI.COMM_WORLD
rank=comm.rank
size=comm.size

MASTER_TO_SLAVE_TAG=1923784#Random unique integer

def exec_in_new_thread(code):
    run_as_new_thread(lambda:exec(code))

def slave_loop():
    while True:
        command=comm.recv(tag=MASTER_TO_SLAVE_TAG)
        #exec_in_new_thread(command)

run_as_new_thread(slave_loop)#Every processor should call this exactly once

def eval_function_from_code(code,function_name,*args,**kwargs):
    #Pass code that genrates a function called function_name
    #It will return the value that function returns when passed *args and **kwargs
    #EXAMPLE:
    # >>> eval_function_from_code('f=lambda x,y:x+y','f',10,y=20)
    # ans = 30

    assert isinstance(function_name,str),'Function name should be a string'
    assert isinstance(code         ,str),'Code should be a string'

    scope=globals()
    exec(code,scope,scope)

    assert function_name in scope,'Your code failed to create a function called '+repr(function_name)
    function=scope[function_name]
    assert callable(function),'Your code created an object called '+repr(function_name)+', but it\'s not a function'

    output=function(*args,**kwargs)
    return output

def accept_command(command):
    #Parse the command object (passed as a dict for mpi4py's convenience)
    assert isinstance(command,dict)
    function_code  =command['function_code']#The source code for the function
    function_name  =command['function_name']#The name of the function
    function_args  =command['args'         ]#args we pass to the function
    function_kwargs=command['kwargs'       ]#kwargs we pass to the fucnction
    reply_dest     =command['reply_dest'   ]#Rank we send the response to
    reply_tag      =command['reply_tag'    ]
    assert function_code  ,str
    assert function_name  ,str
    assert function_args  ,tuple
    assert function_kwargs,dict
    assert reply_dest     ,int
    assert reply_tag      ,int
    assert 0<=reply_rank<size

    #Create the reply
    reply={}#If we don't error, add a 'result' key to reply containing the function result
    try:
        reply['result']=eval_function_from_code(function_code,function_name,*function_args,**function_kwargs)
    finally:
        #No error handling yet...the indication that you have an error is that the reply is empty, and the stack trace this thread prints
        comm.isend(obj=reply,dest=reply_dest,tag=reply_tag)

def 


def parallelized(function):
    #A decorator that will parallelize a function
    function_name=function.__name__
    source_code=get_source_code(function)



def master_command(code):

    assert nodes is None or is_iterable(nodes)
    if nodes is None:nodes=list(range(size))

    for dest in nodes:
        comm.isend(obj =code,
                   dest=dest,
                   tag =MASTER_TO_SLAVE_TAG)


if rank==0:
    pseudo_terminal()
    while True:

        master_command(input('>>>'))


#THIS CODE IS FROM bpython's patch_linecache.py. Get it with pip install bpython.
#I adapted it for use in rp to provide better stack traces

#This module is what lets us do this:
# File <rp-input-4>, line 1, in <module>
#     f()
# File <rp-input-3>, line 2, in f
#     1    def f():
# --> 2        g()

# File <rp-input-3>, line 4, in g
#     3    def g():
# --> 4        h()

# File <rp-input-3>, line 6, in h
#     def h():

# ZeroDivisionError: division by zero

#Note how the stack trace includes lines of code in it, even though they're from user inputs. Previously I needed ipython to do this but I found that bpython also did it. Then I stole bypthon's code and put it in here. lol. Maybe I'll rewrite it some day....when I rewrite rp...

import linecache
class BPythonLinecache(dict):
    """Replaces the cache dict in the standard-library linecache module,
    to also remember (in an unerasable way) rp console input."""

    def __init__(self, *args, **kwargs):
        super(BPythonLinecache, self).__init__(*args, **kwargs)
        self.bpython_history = []

    def is_bpython_filename(self, fname):
        try:
            return fname.startswith('<rp-input-')
        except AttributeError:
            # In case the key isn't a string
            return False

    def get_bpython_history(self, key):
        """Given a filename provided by remember_bpython_input,
        returns the associated source string."""
        try:
            idx = int(key.split('-')[2][:-1])
            return self.bpython_history[idx]
        except (IndexError, ValueError):
            raise KeyError

    def remember_bpython_input(self, source):
        """Remembers a string of source code, and returns
        a fake filename to use to retrieve it later."""
        filename = '<rp-input-%s>' % len(self.bpython_history)
        self.bpython_history.append((len(source), None,
                                     source.splitlines(True), filename))
        return filename

    def __getitem__(self, key):
        if self.is_bpython_filename(key):
            return self.get_bpython_history(key)
        return super(BPythonLinecache, self).__getitem__(key)

    def __contains__(self, key):
        if self.is_bpython_filename(key):
            try:
                self.get_bpython_history(key)
                return True
            except KeyError:
                return False
        return super(BPythonLinecache, self).__contains__(key)

    def __delitem__(self, key):
        if not self.is_bpython_filename(key):
            return super(BPythonLinecache, self).__delitem__(key)


def _bpython_clear_linecache():
    try:
        bpython_history = linecache.cache.bpython_history
    except AttributeError:
        bpython_history = []
    linecache.cache = BPythonLinecache()
    linecache.cache.bpython_history = bpython_history


# Monkey-patch the linecache module so that we're able
# to hold our command history there and have it persist
linecache.cache = BPythonLinecache(linecache.cache)
linecache.clearcache = _bpython_clear_linecache


def filename_for_console_input(code_string):
    """Remembers a string of source code, and returns
    a fake filename to use to retrieve it later."""
    try:
        return linecache.cache.remember_bpython_input(code_string)
    except AttributeError:
        # If someone else has patched linecache.cache, better for code to
        # simply be unavailable to inspect.getsource() than to raise
        # an exception.
        return '<input>'
fc=filename_for_console_input
import code
#ans=code.InteractiveInterpreter().runsource('18/0',fc('18/0'),'single')
interpereter=code.InteractiveInterpreter()
# ans=interpereter.compile('148/0',fc('148/0'),'single')
# def ans():
#     pass
# def run(code):
#     exec(interpereter.compile(code,fc(code),'single'))
# ans=fc
def run_code(code,mode,namespace,exv):
    import sys
    old_tracer=sys.gettrace()#To make sure any debuggers launched in this exeval aren't carried through to pseudo-terminal internal code, while still letting you call pseudo-terminal from a debugger
    try:
        return exv(interpereter.compile(code,fc(code),mode),namespace,namespace)
    finally:#This should happen no matter what, but shouldn't squelch any errors we get when evaluating the code
        #EXIT THE DEBUGGER: (Do this so we don't debug the next pseudo-terminal prompt as well)
        sys.settrace(old_tracer)#Instead of just setting it to None, set it to what it was before. This allows us to run pseudo_terminal from a debugger without exiting the debugger.









"""
Jedi is a static analysis tool for Python that can be used in IDEs/editors.
Jedi has a focus on autocompletion and goto functionality. Jedi is fast and is
very well tested. It understands Python and stubs on a deep level.

Jedi has support for different goto functions. It's possible to search for
usages and list names in a Python file to get information about them.

Jedi uses a very simple API to connect with IDE's. There's a reference
implementation as a `VIM-Plugin <https://github.com/davidhalter/jedi-vim>`_,
which uses Jedi's autocompletion.  We encourage you to use Jedi in your IDEs.
Autocompletion in your REPL is also possible, IPython uses it natively and for
the CPython REPL you have to install it.

Here's a simple example of the autocompletion feature:

>>> import jedi
>>> source = '''
... import json
... json.lo'''
>>> script = jedi.Script(source, 3, len('json.lo'), 'example.py')
>>> script
<Script: 'example.py' ...>
>>> completions = script.completions()
>>> completions
[<Completion: load>, <Completion: loads>]
>>> print(completions[0].complete)
ad
>>> print(completions[0].name)
load

As you see Jedi is pretty simple and allows you to concentrate on writing a
good text editor, while still having very good IDE features for Python.
"""

__version__ = '0.15.1'

from jedi.api import Script, Interpreter, set_debug_function, \
    preload_module, names
from jedi import settings
from jedi.api.environment import find_virtualenvs, find_system_environments, \
    get_default_environment, InvalidPythonEnvironment, create_environment, \
    get_system_environment
from jedi.api.exceptions import InternalError
# Finally load the internal plugins. This is only internal.
from jedi.plugins import registry
del registry
